==> hyades.source_3.0.0_20040621_1225.zip: org/eclipse/hyades/perfmon/PerfmonGenericLoader.java <==
/**********************************************************************
 * Copyright (c) 2003 Hyades project.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors: 
 * Scapa Technologies Limited - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.perfmon;

import org.eclipse.debug.core.model.IProcess;
import org.eclipse.emf.common.notify.impl.*;
import org.eclipse.emf.common.notify.*;

import org.eclipse.hyades.loaders.util.*;

import org.eclipse.emf.common.util.*;

import org.eclipse.hyades.internal.execution.local.common.*;
import org.eclipse.hyades.internal.execution.local.control.*;
import org.eclipse.hyades.internal.execution.local.control.Process;

import org.eclipse.hyades.model.statistical.*;

import org.eclipse.hyades.models.hierarchy.*;
import org.eclipse.hyades.trace.ui.*;
import org.eclipse.hyades.trace.ui.internal.util.ProcessAdapter;

import java.util.*;
import java.io.*;

import org.eclipse.core.runtime.*;

import org.eclipse.swt.widgets.Display;

public class PerfmonGenericLoader extends PerfmonLoader implements DataProcessor, ProcessListener, AgentListener {

private static final boolean PRINTXML = false;	
	
StatisticalFactory factory = StatisticalFactory.eINSTANCE;

long started;
String tracename;
	
Node node;
Process process;
Agent agent;

TRCAgent trcagent;

boolean running = false;

XMLLoader xloader;
ArrayList listeners = new ArrayList();
ModelListener adapter = new ModelListener();

//XMLParser parser = new XMLParser();
//StatisticalXMLTranslator translator;

//OutputStream logfile;
IProgressMonitor pmonitor;

	public PerfmonGenericLoader(IProgressMonitor pmonitor, long started, TRCAgent tagent, String modelname, String host, int port, String remote_host, int freq) throws Exception {
		this.pmonitor = pmonitor;
		this.started = started;
		init(modelname,tagent,host,port,remote_host,freq);
	}
	public PerfmonGenericLoader(long started, TRCAgent tagent, String modelname, String host, String remote_host, int freq) throws Exception {
		this.started = started;
		init(modelname,tagent,host,10002,remote_host,freq);
	}

	public void processLaunched(Process process) {
		//do nothing
	}
	public void processExited(Process process) {
		TRCProcessProxy processProxy = trcagent.getAgentProxy().getProcessProxy();
		if (processProxy == null) { 
            return; 
        } 

		TRCAgentProxy agentProxy = trcagent.getAgentProxy();
        if (agentProxy == null) { 
            return; 
        } 
        
        agentProxy.setActive(false); 
        agentProxy.setAttached(false); 
        agentProxy.setMonitored(false); 
        processProxy.setActive(false); 

//        giveControl(agentProxy, true, false); 
        
        final TRCAgentProxy agentTemp = agentProxy; 

        Display d = Display.getDefault(); 

        d.asyncExec(new Runnable() { 
                public void run() { 
                    //update ui 
                    ProfileEvent event = UIPlugin.getDefault().getProfileEvent(); 

                    event.setSource(agentTemp); 
                    event.setType(ProfileEvent.STOP_MONITOR); 
                    UIPlugin.getDefault().notifyProfileEventListener(event); 
                } 
            }); 
	}
	
	public void agentActive(Agent agent) {
	}
	public void agentInactive(Agent agent) {
		processExited(process);
	}
	public void error(Agent agent, String errorId, String errorMessage) {
	}
	public void handleCommand(Agent agent, CommandElement command) {
	}
	
	public void killProcess() throws Exception {
		node.killProcess(process);
	}
	
	public void setFrequency(int freq) throws IOException {
		setNVPair("var_int", "DC_WAITTIME", ""+freq);
	}

	public void setRemoteHost(String host) throws IOException {
		if (host.length() == 0) host = "NULL";
		PerfmonPlugin.DBG.info("Setting remote host to "+host);
		setNVPair("var_string", "PERFMON_REG_HOST", host);
	}

	public void getUpdatedTree() throws IOException {
		setNVPair("updatetree", "", "true");
	}

	public void getUpdatedTree(SDDescriptor update) throws IOException {
		if (update.getId() == null) return;
		PerfmonPlugin.DBG.info("Requesting tree update for descriptor "+update.getName()+" ("+update.getId()+")");
		setNVPair("updatetree", update.getId(), "true");
	}
	
	public void counterON(SDDescriptor counter) throws IOException, NullPointerException {
		String id = counter.getId();
		if (id == null) throw new NullPointerException("descriptor ID not found");
		setNVPair("filter", id, "true");
	}
	
	public void counterOFF(SDDescriptor counter) throws IOException {
		String id = counter.getId();
		if (id == null) throw new IOException("descriptor ID not found");
		setNVPair("filter", id, "false");
	}

	public SDDescriptor getModel() {
		EList models = trcagent.getDescriptor();
		if (models.size() > 0) { 
			return (SDDescriptor)models.get(0);
		} else {
			return null;	
		}
	}
	
	public void addLoaderListener(StatisticalLoaderListener listener) {
		listeners.add(listener);
	}

	public void removeLoaderListener(StatisticalLoaderListener listener) {
		listeners.add(listener);
	}


	private Agent findAgent(Node node) throws NotConnectedException {
		Agent agent = null;
		Enumeration processes = node.listProcesses();
		
		//
		// Search for an existing agent
		//
		while (processes.hasMoreElements()) {
			Process p = (Process)processes.nextElement();
			PerfmonPlugin.DBG.info("checking RAC process "+p.getName());
			agent = findAgent(p);

			if (agent != null) {
				break;
			}
		}
		
		return agent;
	}

	private Agent findAgent(Process p) {
		if (p == null) return null;
		
		Enumeration agents = p.listAgents();

		Agent the_agent = null;

		if (!agents.hasMoreElements()) {
			PerfmonPlugin.DBG.info("found no agents in this process");
		}
		
		while (agents.hasMoreElements()) {
			Agent a = (Agent)agents.nextElement();

			PerfmonPlugin.DBG.info("found RAC agent "+a.getName()+"/"+a.getType());

			if (a.getName().startsWith("PerfmonAgent")
			||	a.getType().startsWith("PerfmonAgent")) {
				
				try {
//					String agentname = a.getName();
//					agentname = agentname.substring(agentname.indexOf("#")+1);

					if (!a.isAttached()) {
						process = p;
						the_agent = a;
						PerfmonPlugin.DBG.info("agent not attached - accepting");
//						long t = Long.parseLong(agentname); 
//						process = p;
//						agent = a;
						break;
					}
				} catch (Exception e) {
					PerfmonPlugin.DBG.warning("error checking agent "+e);
				}
				
			}
		}
		return the_agent;
	}

	private void init(String modelname, TRCAgent trcagent, String host, int port, String remote_host, int freq) throws Exception {
		
		this.trcagent = trcagent;

		SDView view = (SDView)trcagent.getView();

		//create a view if none exists
		if (view == null) {
			view = factory.createSDView();
			view.setAgent(trcagent);
			view.setName("view");
		}
		
		//create a sample window if none exists
		if (view.eContents().size() == 0) {
			SDSampleWindow window = factory.createSDSampleWindow();
			window.setView(view);
		}
	
		//add EMF listeners to the agent and the all the sample windows (should catch adds for descriptors and observations)
		trcagent.eAdapters().add(adapter);
		trcagent.getView().eAdapters().add(adapter);
		
		EList windows = view.eContents();
		for (int k = 0; k < windows.size(); k++) {
			SDSampleWindow window = (SDSampleWindow)windows.get(k);
			window.eAdapters().add(adapter);
		}


//logfile = new FileOutputStream("C:\\XMLLOG.txt");
				
		tracename = modelname;

		xloader = new XMLLoader(trcagent);		

		PerfmonPlugin.DBG.warning("asked to connect to "+host+":"+port);

		try {
			
			node = NodeFactory.createNode(host);
			
			PerfmonPlugin.DBG.info("connecting to RAC");
			
			if (pmonitor != null) pmonitor.setTaskName(PerfmonPlugin.getString("PROGRESS_CONNECTING"));
			node.connect(port);

			PerfmonPlugin.DBG.info("connected to RAC");

			if (pmonitor != null) pmonitor.setTaskName(PerfmonPlugin.getString("PROGRESS_CREATING"));

			//
			// Find an agent
			//			
			agent = findAgent(node); 
			
			//
			// No agent found, try to launch one
			//			
			if (agent == null) {
				PerfmonPlugin.DBG.info("Launching RAC PerfmonAgent");
				Process process = ProcessFactory.createProcess(node,"PerfmonAgent");
				
				 
		            
				if (process != null) {
					final IProcess adapter = new ProcessAdapter(process, null);
		            process.addProcessListener(new ProcessListener() {
		             
		             public void processLaunched(Process p)
		             {
		             	UIPlugin.getDefault().registerLaunchProcess(adapter);
		             }
		             
		             public void processExited(Process p)
		             {
		             	UIPlugin.getDefault().deregisterLaunchProcess(adapter);
		             }
		            });
		            
		            ProcessLaunch launcher = new ProcessLaunch();
		            launcher.process = process;
		            Display display = Display.getCurrent();
		            if (display == null) display = Display.getDefault();
		            display.syncExec(launcher);
//					process.launch();

					long t = System.currentTimeMillis() + 8000;	//search for a maximum of 25 seconds

					//wait for the process to become active
					while (!process.isActive() && System.currentTimeMillis() < t) {
						try {
							Thread.sleep(200);	
						} catch (Exception e) {
						}
					}

					t = System.currentTimeMillis() + 25000;	//search for a maximum of 25 seconds
					agent = findAgent(node); 
					while (agent == null && System.currentTimeMillis() < t) {
						Thread.sleep(1250);
						agent = findAgent(node);
					} 
				}
			}

//			if (agent == null) {
//				PerfmonPlugin.DBG.info("Disconnecting from node and trying to find agent again");
//				node.getConnection().disconnect();
//				node.connect(port);
//
//				agent = findAgent(node); 
//			}

			//
			// If we now have an agent, 
			//
			if (agent != null) {
				if (pmonitor != null) pmonitor.setTaskName(PerfmonPlugin.getString("PROGRESS_CONFIGURING"));

				agent.addAgentListener(this);
				process.addProcessListener(this);
				
				PerfmonPlugin.DBG.info("attaching to perfmon agent "+agent.getName());

				running = true;

				try {
					trcagent.getAgentProxy().getProcessProxy().setPid(Integer.parseInt(agent.getProcess().getProcessId()));
				} catch (Exception e) {
					PerfmonPlugin.DBG.error("could not set process proxy PID");
				}
	
				trcagent.getAgentProxy().setRuntimeId(process.getUUID());
				
				LoadersUtils.registerAgentInstance(trcagent.getAgentProxy(),agent);
				
				agent.attach();
				agent.startMonitoring(this);
				trcagent.setRuntimeId(agent.getUUID());

				PerfmonPlugin.DBG.info("attached To & Monitoring Perfmon Agent");
	
				setRemoteHost(remote_host);
				setFrequency(freq);
				getUpdatedTree();
				
				long t = System.currentTimeMillis();
				
				while (this.getModel() == null) {
					try {
						Thread.sleep(100);
					} catch (Exception e){}	
					if (System.currentTimeMillis()-t > 10000) {
						t = Long.MAX_VALUE;
						PerfmonPlugin.DBG.info("been waiting 10 seconds and still no model...");
					}
				}
								
			} else {
				PerfmonPlugin.DBG.warning("no perfmon agent found");
				throw new Exception("Could not create PerfmonAgent on RAC");
			}
			
		} catch (Exception e) {
			PerfmonPlugin.DBG.warning("problem initialising loader");

//////////////// FOR SAMPLE EVENTS ONLY
//temporarily disabled for sample events
			throw e;
////////////////////////////////////////

		}
	}
	
	class ProcessLaunch implements Runnable{
		Process process;
		public void run() {
			try {
				process.launch();
			} catch (Exception e) {
				PerfmonPlugin.DBG.logVisibleError(e,PerfmonPlugin.getString("ERROR_LAUNCH_PERFMON"),true);
			}
		}
	}

	public void setNVPair(String type, String name, String value) throws IOException {
		
		ControlMessage message=new ControlMessage();
		SetNVPairCommand command=new SetNVPairCommand();
		try {
			command.setProcessId(Long.parseLong(process.getProcessId()));
		}
		catch(InactiveProcessException e) {
			throw new IOException("Inactive process - try again soon?");
		}
		command.setAgentName(agent.getName());
		command.setType(type);
		command.setName(name);
		command.setValue(value);
		message.appendCommand(command);
		try {
			node.getConnection().sendMessage(message, null);
		} catch(IOException e) {
			//log errors?
			throw e;
		}
	}

	public boolean isRunning() {
		return running;
	}

	public void shutdown() {
		running = false;
		PerfmonPlugin.DBG.info("shutdown");
		if (agent != null) {
			try {
				PerfmonPlugin.DBG.info("detaching from agent");
				agent.stopMonitoring();
				agent.detach();	
				
			} catch (Exception e) {
				PerfmonPlugin.DBG.warning("failed to detach from agent",e);
			}
		}

	}

	protected void finalize() throws Throwable {
		super.finalize();
		shutdown();
	}

	/////////////////////////////////////////////
	//  DATA PROCESSOR IMPL
	/////////////////////////////////////////////

	public void incoming(String msg) {
		try {
//			logfile.write(msg.getBytes());
		} catch (Throwable t) {
t.printStackTrace();			
			PerfmonPlugin.DBG.warning("unable to parse message "+msg);
		}
	}

	private byte[] stringToBytes(String s) {
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		for (int i = 0; i < s.length(); i++) {
			bout.write((byte)s.charAt(i));	
		}
		return bout.toByteArray();
	}

boolean first = true;
	public void incommingData(byte[] buffer, int length, java.net.InetAddress peer) {

		buffer = new String(buffer,0,length).trim().getBytes();
		length = buffer.length;

		if (length == 0) return;

		if (first) {
			byte[] ROOT_START = "<TRACE>".getBytes();
			xloader.loadEvent(ROOT_START,ROOT_START.length);
			first = false;	
		} 

		try {			
//logfile.write(buffer,0,length);
if (PRINTXML) System.out.println(new String(buffer,0,length));			
			
			xloader.loadEvent(buffer,length);			
		} catch (Throwable t) {
			PerfmonPlugin.DBG.error("unable to parse message "+new String(buffer,0,30)+"...",t);
		}

//how many fragments has the loader loaded?
//		try {
//			Field field = XMLLoader.class.getDeclaredField("fragmentsCount");
//			field.setAccessible(true);
//			PerfmonPlugin.DBG.info("LOADED FRAGMENTS = "+field.getInt(xloader));
//		} catch (Exception e) {
//			e.printStackTrace();			
//		}

//check the model for changes
//		EList models = trcagent.getDescriptor();
//		for (int i = 0; i < models.size(); i++) {
//			checkModel((SDDescriptor)models.get(i));
//		}
	}

	public void incommingData(char[] buffer, int length, java.net.InetAddress peer) {
		//?
	}


	public void invalidDataType(byte[] data, int length, java.net.InetAddress peer) {
		PerfmonPlugin.DBG.warning("invalid datatype");
	}

	public void waitingForData() {
	}
	
	class ModelListener extends AdapterImpl {
		public void notifyChanged(Notification notification) {
//			PerfmonPlugin.DBG.info("NOTIFICATION RECEIVED "+notification);
			if(notification.getEventType() == Notification.ADD) {
				Object newValue = notification.getNewValue();

				boolean notify = false;
				SDDescriptor parent = null;

				if (newValue instanceof SDDescriptor) {
					notify = true;
					parent = ((SDDescriptor)newValue).getParent();
				} else if (newValue instanceof SDSnapshotObservation) {
					notify = true;
					parent = ((SDSnapshotObservation)newValue).getMemberDescriptor();
				}				
				
				if (notify) {
					for (int i = 0; i < listeners.size(); i++) {
						StatisticalLoaderListener listener = (StatisticalLoaderListener)listeners.get(i);
						listener.modelChanged(parent);	
					}
				}
			}
		}	
	}
}

==> hyades.source_3.0.2_20050225_1641.zip: org/eclipse/hyades/perfmon/PerfmonGenericLoader.java <==
/**********************************************************************
 * Copyright (c) 2003,2004 Scapa Technologies Limited and others
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors: 
 * Scapa Technologies Limited - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.perfmon;

import org.eclipse.debug.core.model.IProcess;
import org.eclipse.emf.common.notify.impl.*;
import org.eclipse.emf.common.notify.*;

import org.eclipse.hyades.loaders.util.*;

import org.eclipse.emf.common.util.*;

import org.eclipse.hyades.internal.execution.local.common.*;
import org.eclipse.hyades.internal.execution.local.control.*;
import org.eclipse.hyades.internal.execution.local.control.Process;

import org.eclipse.hyades.model.statistical.*;

import org.eclipse.hyades.models.hierarchy.*;
import org.eclipse.hyades.trace.ui.*;
import org.eclipse.hyades.trace.ui.internal.util.ProcessAdapter;

import java.util.*;
import java.io.*;

import org.eclipse.core.runtime.*;

import org.eclipse.swt.widgets.Display;

public class PerfmonGenericLoader extends PerfmonLoader implements DataProcessor, ProcessListener, AgentListener {

private static final boolean PRINTXML = false;	
private boolean LOGXML = false;	
	
StatisticalFactory factory = StatisticalFactory.eINSTANCE;

long started;
String tracename;
	
Node node;
Process process;
Agent agent;

TRCAgent trcagent;

boolean running = false;

XMLLoader xloader;
ArrayList listeners = new ArrayList();
ModelListener adapter = new ModelListener();

//XMLParser parser = new XMLParser();
//StatisticalXMLTranslator translator;

OutputStream logfile;
IProgressMonitor pmonitor;

	public PerfmonGenericLoader(IProgressMonitor pmonitor, long started, TRCAgent tagent, String modelname, String host, int port, String remote_host, int freq) throws Exception {
		this.pmonitor = pmonitor;
		this.started = started;
		init(modelname,tagent,host,port,remote_host,freq);
	}
	public PerfmonGenericLoader(long started, TRCAgent tagent, String modelname, String host, String remote_host, int freq) throws Exception {
		this.started = started;
		init(modelname,tagent,host,10002,remote_host,freq);
	}

	public void processLaunched(Process process) {
		//do nothing
	}
	public void processExited(Process process) {
		TRCProcessProxy processProxy = trcagent.getAgentProxy().getProcessProxy();
		if (processProxy == null) { 
            return; 
        } 

		TRCAgentProxy agentProxy = trcagent.getAgentProxy();
        if (agentProxy == null) { 
            return; 
        } 
        
        agentProxy.setActive(false); 
        agentProxy.setAttached(false); 
        agentProxy.setMonitored(false); 
        processProxy.setActive(false); 

//        giveControl(agentProxy, true, false); 
        
        final TRCAgentProxy agentTemp = agentProxy; 

        Display d = Display.getDefault(); 

        d.asyncExec(new Runnable() { 
                public void run() { 
                    //update ui 
                    ProfileEvent event = UIPlugin.getDefault().getProfileEvent(); 

                    event.setSource(agentTemp); 
                    event.setType(ProfileEvent.STOP_MONITOR); 
                    UIPlugin.getDefault().notifyProfileEventListener(event); 
                } 
            }); 
	}
	
	public void agentActive(Agent agent) {
	}
	public void agentInactive(Agent agent) {
		processExited(process);
	}
	public void error(Agent agent, String errorId, String errorMessage) {
	}
	public void handleCommand(Agent agent, CommandElement command) {
	}
	
	public void killProcess() throws Exception {
		node.killProcess(process);
	}
	
	public void setFrequency(int freq) throws IOException {
		setNVPair("var_int", "DC_WAITTIME", ""+freq);
	}

	public void setRemoteHost(String host) throws IOException {
		if (host.length() == 0) host = "NULL";
		PerfmonPlugin.DBG.info("Setting remote host to "+host);
		setNVPair("var_string", "PERFMON_REG_HOST", host);
	}

	public void getUpdatedTree() throws IOException {
		setNVPair("updatetree", "", "true");
	}

	public void getUpdatedTree(SDDescriptor update) throws IOException {
		if (update.getId() == null) return;
		PerfmonPlugin.DBG.info("Requesting tree update for descriptor "+update.getName()+" ("+update.getId()+")");
		setNVPair("updatetree", update.getId(), "true");
	}
	
	public void counterON(SDDescriptor counter) throws IOException, NullPointerException {
		String id = counter.getId();
		if (id == null) throw new NullPointerException(PerfmonPlugin.getString("ERROR_DESCRIPTOR_ID_NOT_FOUND"));
		setNVPair("filter", id, "true");
	}
	
	public void counterOFF(SDDescriptor counter) throws IOException {
		String id = counter.getId();
		if (id == null) throw new IOException(PerfmonPlugin.getString("ERROR_DESCRIPTOR_ID_NOT_FOUND"));
		setNVPair("filter", id, "false");
	}

	public SDDescriptor getModel() {
		EList models = trcagent.getDescriptor();
		if (models.size() > 0) { 
			return (SDDescriptor)models.get(0);
		} else {
			return null;	
		}
	}
	
	public void addLoaderListener(StatisticalLoaderListener listener) {
		listeners.add(listener);
	}

	public void removeLoaderListener(StatisticalLoaderListener listener) {
		listeners.add(listener);
	}


	private Agent findAgent(Node node) throws NotConnectedException {
		Agent agent = null;
		Enumeration processes = node.listProcesses();
		
		//
		// Search for an existing agent
		//
		while (processes.hasMoreElements()) {
			Process p = (Process)processes.nextElement();
			PerfmonPlugin.DBG.info("checking RAC process "+p.getName());
			agent = findAgent(p);

			if (agent != null) {
				break;
			}
		}
		
		return agent;
	}

	private Agent findAgent(Process p) {
		if (p == null) return null;
		
		Enumeration agents = p.listAgents();

		Agent the_agent = null;

		if (!agents.hasMoreElements()) {
			PerfmonPlugin.DBG.info("found no agents in this process");
		}
		
		while (agents.hasMoreElements()) {
			Agent a = (Agent)agents.nextElement();

			PerfmonPlugin.DBG.info("found RAC agent "+a.getName()+"/"+a.getType());

			if (a.getName().startsWith("PerfmonAgent")
			||	a.getType().startsWith("PerfmonAgent")) {
				
				try {
//					String agentname = a.getName();
//					agentname = agentname.substring(agentname.indexOf("#")+1);

					if (!a.isAttached()) {
						process = p;
						the_agent = a;
						PerfmonPlugin.DBG.info("agent not attached - accepting");
//						long t = Long.parseLong(agentname); 
//						process = p;
//						agent = a;
						break;
					}
				} catch (Exception e) {
					PerfmonPlugin.DBG.warning("error checking agent "+e);
				}
				
			}
		}
		return the_agent;
	}

	private void init(String modelname, TRCAgent trcagent, String host, int port, String remote_host, int freq) throws Exception {
		
		this.trcagent = trcagent;

		SDView view = (SDView)trcagent.getView();

		//create a view if none exists
		if (view == null) {
			view = factory.createSDView();
			view.setAgent(trcagent);
			view.setName("view");
		}
		
		//create a sample window if none exists
		if (view.eContents().size() == 0) {
			SDSampleWindow window = factory.createSDSampleWindow();
			window.setView(view);
		}
	
		//add EMF listeners to the agent and the all the sample windows (should catch adds for descriptors and observations)
		trcagent.eAdapters().add(adapter);
		trcagent.getView().eAdapters().add(adapter);
		
		EList windows = view.eContents();
		for (int k = 0; k < windows.size(); k++) {
			SDSampleWindow window = (SDSampleWindow)windows.get(k);
			window.eAdapters().add(adapter);
		}


if (LOGXML) {
	try {
		logfile = new FileOutputStream("C:\\temp\\XMLLOG.txt");
	} catch (Throwable t) {}
}
				
		tracename = modelname;

		xloader = new XMLLoader(trcagent);		

		PerfmonPlugin.DBG.warning("asked to connect to "+host+":"+port);

		try {
			
			node = NodeFactory.createNode(host);
			
			PerfmonPlugin.DBG.info("connecting to RAC");
			
			if (pmonitor != null) pmonitor.setTaskName(PerfmonPlugin.getString("PROGRESS_CONNECTING"));
			node.connect(port);

			PerfmonPlugin.DBG.info("connected to RAC");

			if (pmonitor != null) pmonitor.setTaskName(PerfmonPlugin.getString("PROGRESS_CREATING"));

			//
			// Find an agent
			//			
			agent = findAgent(node); 
			
			//
			// No agent found, try to launch one
			//			
			if (agent == null) {
				PerfmonPlugin.DBG.info("Launching RAC PerfmonAgent");
				Process process = ProcessFactory.createProcess(node,"PerfmonAgent");
				
				 
		            
				if (process != null) {
					final IProcess adapter = new ProcessAdapter(process, null);
		            process.addProcessListener(new ProcessListener() {
		             
		             public void processLaunched(Process p)
		             {
		             	UIPlugin.getDefault().registerLaunchProcess(adapter);
		             }
		             
		             public void processExited(Process p)
		             {
		             	UIPlugin.getDefault().deregisterLaunchProcess(adapter);
		             }
		            });
		            
		            ProcessLaunch launcher = new ProcessLaunch();
		            launcher.process = process;
		            Display display = Display.getCurrent();
		            if (display == null) display = Display.getDefault();
		            display.syncExec(launcher);
//					process.launch();

					long t = System.currentTimeMillis() + 8000;	//search for a maximum of 25 seconds

					//wait for the process to become active
					while (!process.isActive() && System.currentTimeMillis() < t) {
						try {
							Thread.sleep(200);	
						} catch (Exception e) {
						}
					}

					t = System.currentTimeMillis() + 25000;	//search for a maximum of 25 seconds
					agent = findAgent(node); 
					while (agent == null && System.currentTimeMillis() < t) {
						Thread.sleep(1250);
						agent = findAgent(node);
					} 
				}
			}

//			if (agent == null) {
//				PerfmonPlugin.DBG.info("Disconnecting from node and trying to find agent again");
//				node.getConnection().disconnect();
//				node.connect(port);
//
//				agent = findAgent(node); 
//			}

			//
			// If we now have an agent, 
			//
			if (agent != null) {
				if (pmonitor != null) pmonitor.setTaskName(PerfmonPlugin.getString("PROGRESS_CONFIGURING"));

				agent.addAgentListener(this);
				process.addProcessListener(this);
				
				PerfmonPlugin.DBG.info("attaching to perfmon agent "+agent.getName());

				running = true;

				try {
					trcagent.getAgentProxy().getProcessProxy().setPid(Integer.parseInt(agent.getProcess().getProcessId()));
				} catch (Exception e) {
					PerfmonPlugin.DBG.error("could not set process proxy PID");
				}
	
				trcagent.getAgentProxy().setRuntimeId(process.getUUID());
				
				LoadersUtils.registerAgentInstance(trcagent.getAgentProxy(),agent);
				
				agent.attach();
				agent.startMonitoring(this);
				trcagent.setRuntimeId(agent.getUUID());

				PerfmonPlugin.DBG.info("attached To & Monitoring Perfmon Agent");
	
				setRemoteHost(remote_host);
				setFrequency(freq);
				getUpdatedTree();
				
				long t = System.currentTimeMillis();
				
				while (this.getModel() == null) {
					try {
						Thread.sleep(100);
					} catch (Exception e){}	
					if (System.currentTimeMillis()-t > 10000) {
						t = Long.MAX_VALUE;
						PerfmonPlugin.DBG.info("been waiting 10 seconds and still no model...");
					}
				}
								
			} else {
				PerfmonPlugin.DBG.warning("no perfmon agent found");
				throw new Exception(PerfmonPlugin.getString("ERROR_CREATE_PERFMON_AGENT"));
			}
			
		} catch (Exception e) {
			PerfmonPlugin.DBG.warning("problem initialising loader");

//////////////// FOR SAMPLE EVENTS ONLY
//temporarily disabled for sample events
			throw e;
////////////////////////////////////////

		}
	}
	
	class ProcessLaunch implements Runnable{
		Process process;
		public void run() {
			try {
				process.launch();
			} catch (Exception e) {
				PerfmonPlugin.DBG.logVisibleError(e,PerfmonPlugin.getString("ERROR_LAUNCH_PERFMON"),true);
			}
		}
	}

	public void setNVPair(String type, String name, String value) throws IOException {
		
		ControlMessage message=new ControlMessage();
		SetNVPairCommand command=new SetNVPairCommand();
		try {
			command.setProcessId(Long.parseLong(process.getProcessId()));
		}
		catch(InactiveProcessException e) {
			throw new IOException(PerfmonPlugin.getString("ERROR_INACTIVE_PROCESS"));
		}
		command.setAgentName(agent.getName());
		command.setType(type);
		command.setName(name);
		command.setValue(value);
		message.appendCommand(command);
		try {
			node.getConnection().sendMessage(message, null);
		} catch(IOException e) {
			//log errors?
			throw e;
		}
	}

	public boolean isRunning() {
		return running;
	}

	public void shutdown() {
		running = false;
		PerfmonPlugin.DBG.info("shutdown");
		if (agent != null) {
			try {
				PerfmonPlugin.DBG.info("detaching from agent");
				agent.stopMonitoring();
				agent.detach();	
				
			} catch (Exception e) {
				PerfmonPlugin.DBG.warning("failed to detach from agent",e);
			}
		}

	}

	protected void finalize() throws Throwable {
		super.finalize();
		shutdown();
	}

	/////////////////////////////////////////////
	//  DATA PROCESSOR IMPL
	/////////////////////////////////////////////

	public void incoming(String msg) {
		try {
//			logfile.write(msg.getBytes());
		} catch (Throwable t) {
t.printStackTrace();			
			PerfmonPlugin.DBG.warning("unable to parse message "+msg);
		}
	}

	private byte[] stringToBytes(String s) {
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		for (int i = 0; i < s.length(); i++) {
			bout.write((byte)s.charAt(i));	
		}
		return bout.toByteArray();
	}

boolean first = true;
	public void incommingData(byte[] buffer, int length, java.net.InetAddress peer) {

if (LOGXML) {		
	try {
		logfile.write(buffer,0,length);
	} catch (Throwable t) {
		LOGXML = false;
	}
}
		
		buffer = new String(buffer,0,length).trim().getBytes();
		length = buffer.length;

		if (length == 0) return;

		if (first) {
			byte[] ROOT_START = "<TRACE>".getBytes();
			xloader.loadEvent(ROOT_START,ROOT_START.length);
			first = false;	
		} 

		try {			
//logfile.write(buffer,0,length);
if (PRINTXML) System.out.println(new String(buffer,0,length));			
			
			xloader.loadEvent(buffer,length);			
		} catch (Throwable t) {
			PerfmonPlugin.DBG.error("unable to parse message "+new String(buffer,0,30)+"...",t);
		}

//how many fragments has the loader loaded?
//		try {
//			Field field = XMLLoader.class.getDeclaredField("fragmentsCount");
//			field.setAccessible(true);
//			PerfmonPlugin.DBG.info("LOADED FRAGMENTS = "+field.getInt(xloader));
//		} catch (Exception e) {
//			e.printStackTrace();			
//		}

//check the model for changes
//		EList models = trcagent.getDescriptor();
//		for (int i = 0; i < models.size(); i++) {
//			checkModel((SDDescriptor)models.get(i));
//		}
	}

	public void incommingData(char[] buffer, int length, java.net.InetAddress peer) {
		//?
	}


	public void invalidDataType(byte[] data, int length, java.net.InetAddress peer) {
		PerfmonPlugin.DBG.warning("invalid datatype");
	}

	public void waitingForData() {
	}
	
	class ModelListener extends AdapterImpl {
		public void notifyChanged(Notification notification) {
//			PerfmonPlugin.DBG.info("NOTIFICATION RECEIVED "+notification);
			if(notification.getEventType() == Notification.ADD) {
				Object newValue = notification.getNewValue();

				boolean notify = false;
				SDDescriptor parent = null;

				if (newValue instanceof SDDescriptor) {
					notify = true;
					parent = ((SDDescriptor)newValue).getParent();
				} else if (newValue instanceof SDSnapshotObservation) {
					notify = true;
					parent = ((SDSnapshotObservation)newValue).getMemberDescriptor();
				}				
				
				if (notify) {
					for (int i = 0; i < listeners.size(); i++) {
						StatisticalLoaderListener listener = (StatisticalLoaderListener)listeners.get(i);
						listener.modelChanged(parent);	
					}
				}
			}
		}	
	}
}
